;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;    BeeLine Communications Interface
;    Copyright (C) 2005  BigRedBee, LLC
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    BigRedBee, LLC
;    5752 Bay Point Dr
;    Lake Oswego, OR  97035
;
;    info@bigredbee.com
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define version "0105"

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
list        p=16F688        ; list directive to define processor
#include    <p16f688.inc>   ; processor specific variable definitions

ver equ 688
errorlevel  -302            ; suppress message 302 from list file
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Changelog
;
; version 0105
;  - minor cosmetic changes and code cleanup, no functional changes
;
; version 0104
; - call ad_convert early in cmd_text_dump to make sure we send valid data
; - wait before doing ad conversion in check_shutdown. to allow v to creep
;   back up
; - set ADRESULTHI and LO to 0's, not 5A.  Looks better when reading 
;   prioir to doing AD conversion
;
; Version 0103
; - When zeroing flash, zero all 3 bytes
; - when writing time to flash, write all 3 bytes.
;
; Version 0102
; - check for input characters in between each morse character transmission
;   make sure to poweroff and poweron the transmitter appropriately
;
; Version 0101
; - instead of goto after cmd_dispatch detects a bogus character, do a return
;   makes plugging and unplugging (which generates spurious chars) transparent
; - check for characters inside the mores loop
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define model "BLTX0"

#define SER_DRIVER 1		; set to 0 if driving serial bus direct
#define SIMULATE 0		; for debug
#define INTERRUPTS_ENABLED 1	; for debug

__CONFIG  _INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOD_OFF & _FCMEN_OFF  & _IESO_OFF  

;__CONFIG  _INTRC_OSC_NOCLKOUT & _WDT_OFF &  _PWRTE_OFF & _MCLRE_ON  & _CP_OFF & _BOD_OFF & _FCMEN_OFF  & _IESO_OFF  

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Pins definitions:
;

#define cc_pale    PORTA, 5		;; output
#define cc_pdata   PORTA, 4		;; output	
#define mclr_NA    PORTA, 3		;; input ONLY
#define cc_pclk    PORTA, 2		;; output
#define serout_pin PORTA, 1		;; output
#define serin_pin  PORTA, 0		;; input

#define TRIS_PORTA B'001001'

#define rc5_NA     PORTC, 5		;; input		
#define rc4_NA     PORTC, 4		;; input
#define rc3_NA     PORTC, 3		;; input
#define cc_di      PORTC, 2		;; output
#define cc_dclk    PORTC, 1		;; output	UNUSED
#define vbat       PORTC, 0		;; input AN4

#define TRIS_PORTC  B'0111001'

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Defines
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

w	equ	0x00 
f	equ	0x01 
pc	equ	0x02 
status	equ	0x03 
zerof	equ	0x02 
caryf	equ	0x00 

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Bank Select Defines
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	Bank0		bcf	STATUS, RP0
#define	Bank1		bsf	STATUS, RP0

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;includes
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "macros.inc"

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; This block of data gets transferred back and forth over the serial
; port 
;
; Some entries are unused:
; - beep_space_lv
; - beep_freq_loop_h
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	cblock  0x20

        callsign: 0x20		;  message area
	f2			;  Frequency
	f1
	f0
	power_out		; power amplifier setting
	beep_tone		; freq (tone) of beep
	beep_len		; length of beep
	beep_num		; number of beeps between morse
	beep_space		; time between beeps
	beep_freq_h		; Beep pitch high
	beep_freq		; Beep pitch
	beep_freq_loop_h	; timing loop
	beep_freq_loop          ; timing loop
        morse_freq_h            ; morse pitch high
	morse_freq		; morse pitch
	morse_freq_loop		; timing loop;
	beep_space_lv	        ; time between beeps when voltage is low
	options

	v_cutoffh		; trigger for low V operation
	v_cutoff		; trigger for low V operation

	time2			; elapsed time
	time1
	time0
	ADRESULTHI		; A/D Conversion result
	ADRESULTLO		 

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; This is the end of the image that gets transferred on a "read" command
; 
; general purpose variables follow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	WTemp
	outdata
	outaddr
	StatusTemp
	
	delay1 
	msgreg0
	msgreg1
	didah
	send
	
	delay2
	d100_h
	d100
	d250_h
	d250
	d5
	i:2
	j
	k
	pause_i
        f_divhi		; for the divide routines
        f_divlo		
	clk_state
endc

program_length equ  ADRESULTLO - callsign + 1
program_pgm_length equ program_length -5 

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; cc0105f:  Read from flash, write to cc1050;

cc1050f MACRO addr0, eeaddr0,  
	movlw addr0
	movwf outaddr

	movlw eeaddr0

	banksel EEADR
;	bsf status, RP0
;	bcf status, RP1
	movwf EEADR
	bcf EECON1, EEPGD
	bsf EECON1, RD

	movf EEDAT, w

	banksel outdata
	movwf outdata

	call write_addr_data
    endm

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; cc0105r:  Read from register, write to cc1050;

cc1050r MACRO addr0, raddr0,  
	movlw addr0
	movwf outaddr

	movfw raddr0
	movwf outdata

	call write_addr_data
    endm

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; cc0105:  Read from immediate, write to cc1050;

cc1050 MACRO addr0, data0 
;
	movlw addr0
	movwf outaddr
	movlw data0
	movwf outdata
	call write_addr_data
    endm

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Begin Program Memory
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	ORG		0x000		;RESET Vector
	nop				;for ICD use
	goto		S0		;Initialize the chip in State 0

	ORG		0x004		;Interrupt Vector
	movwf		WTemp		;Save W register
	swapf		STATUS, W	;Swap status to be saved into W
	movwf		StatusTemp	;Save STATUS register
;
; DISABLE INTERRUPTS
;
	bcf  INTCON, 6
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	ifbs INTCON, RAIF
	
	call ser_in
	bcf INTCON, RAIF
	endif_
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	ifbs PIR1, 0
	call clkint
	bcf PIR1, 0
	endif_
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
; re-enable Interrupts
;
	bsf  INTCON, 6

	swapf	StatusTemp, W	; swap status_temp into W, sets Bank to original state
	movwf	STATUS		; restore STATUS register
	swapf	WTemp, F
	swapf	WTemp, W	; restore W register
	retfie

;=========================================================== 
;	w contains a number 0 to 9, returns morse 
;       character for w

msg		
	movwf FSR
	movfw INDF
	return

	addwf   pc,f        

  

get_version
Table
	addwf PCL, f
	dt version, 0
TableEnd
get_model
	addwf PCL, F
	dt model, 0




msgnum
	addwf pc, f
	retlw m0
	retlw m1
	retlw m2
	retlw m3
	retlw m4
	retlw m5
	retlw m6
	retlw m7
	retlw m8
	retlw m9


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Subroutine: S0, State 0
;
;Description: Initialize the chip
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
S0

;PortA/C Init code
	banksel TRISA

	MOVLW TRIS_PORTA		; 1 means input
	MOVWF TRISA			; 


	MOVLW TRIS_PORTC		; 1 means input
	MOVWF TRISC	

	banksel OPTION_REG
	bcf	OPTION_REG, 7

	banksel WPUA
	bsf	WPUA, 0
	
	banksel PORTA	; 
        CLRF PORTA
	CLRF PORTC


	MOVLW 0x07			; Comparators off;
	MOVWF CMCON0

	BSF STATUS, RP0
	MOVLW 0x10
	MOVWF ANSEL			; AN4 (C0) is an analog input
	BCF STATUS, RP0

	call read_program


;	clrf time0
;	clrf time1
	clrf clk_state

; Set AD to zero, so if we read before we convert, we get 0.
;
	MOVLW 0x00
	MOVWF ADRESULTHI
	MOVWF ADRESULTLO

#if SER_DRIVER
	bsf		serout_pin; for use w/o inverted
#else
	bcf 		serout_pin;
#endif

; this code is unnecessary !!!
	#if 1
	call ser_enable
        bcf ser_con, SER_READY
	#endif

;
; before we do ANYTHING, sit here for 5 seconds and wait for an input character
;

	for k, 0, 50
   	  call	w100mS		; wait 100mS 
	  banksel ser_con
again_h1
	  btfsc ser_con, SER_READY
	  goto cmd_dispatch_h1	; char available
	  goto continue10		; no char


cmd_dispatch_h1
	  call cmd_dispatch
	  goto again_h1			; if illegal character, discard

continue10
	next

;;;;;;
;
; Enable timer interrupts if options allow

	btfsc	options,3	; SKIP timer init if set.
	goto beacon_start

	movlw 0x31		
	movwf T1CON		; 8/1 pre-scaler

	bsf INTCON, 6

	banksel PIE1
	bsf PIE1, 0
	banksel TMR0

	goto beacon_start

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

cmd_dispatch

	call ser_get
	bank00
	switch 

	  case 'M'
	  	goto cmd_model
	  endcase			

	  case 'V'
	  	goto cmd_version
	  endcase			

	  case 'T'
	  	goto cmd_text_dump
	  endcase			

	  case 'P'
	  	goto cmd_text_pgm
	  endcase

	  case 'G'
	  	;goto S0
		goto beacon_start
	  endcase

	  case 'S'
	  	goto cmd_read_serialnum
	  endcase

	  case 'C'
	  	goto cmd_read_callsign
	  endcase

	  case 'Z'
	  	goto cmd_zero_time
	  endcase

          case 'U'
	  	goto cmd_update_serial
	  endcase

; what do we do if we get a bogus character?  Let's start all over again.
; 
; no wait, let's just return

	  return

   	  goto S0
	  endswitch

cmd_read_serialnum

	bank0
	for k, serialnum, serialnum+4
		movfw k
		banksel EEADR
		movwf 	EEADR
		bank00
		call flash_read
		call ser_out
	next
	goto cmd_ok

cmd_read_callsign
	bank0
	for k, scall, scall+5
		movfw k
		banksel EEADR
		movwf	EEADR
	bank00
		call flash_read
		call ser_out
	next
	goto cmd_ok

cmd_zero_time
	for k, eetime2, eetime0
		movfw k
		banksel EEADR
		movwf EEADR
		bank00
		movlw 0
		call flash_write
	next
	bank00
	movlw 0
	movwf time2
	movwf time1
	movwf time0
	goto cmd_ok
		


;; the default here should be to go to S0 if we get an illegal character.

cmd_update_serial

	movlw 0x0a 
	call ser_out
	movlw  0x0d
	call ser_out
	bank00

	banksel callsign
	movlw callsign
	movwf FSR

	for i,0, 9		;;6 callsign charas + 4 sernum
		banksel ser_con
		btfss ser_con, SER_READY
		goto $-1

		call ser_get	
		bank00
		movwf INDF
		incf FSR, f
    next	
	call write_serial
	call read_program	;now restore the data we trashed
	goto cmd_ok

cmd_text_pgm

	movlw 0x0a 
	call ser_out
	movlw  0x0d
	call ser_out
	bank00

	banksel callsign
	movlw callsign
	movwf FSR

	for i,0, program_pgm_length-1
		banksel ser_con
		btfss ser_con, SER_READY
		goto $-1

		call ser_get	
		bank00
		movwf INDF
		incf FSR, f
        next	
	call write_program
	goto cmd_ok


cmd_text_dump

        call ad_convert	;; this used to be just before goto cmd_ok
			;; need to move here to guarantee valid values

	banksel callsign

	movlw callsign
	movwf FSR

	for i, 0, program_length-1
	movfw INDF
	call ser_out
	incf FSR, F
	next

	goto cmd_ok


cmd_version
	call output_version
	goto cmd_ok

cmd_model
	call output_model
	goto cmd_ok

cmd_ok
	movlw 0x0a 
	call ser_out
	movlw  0x0d
	call ser_out
	bank00

get_cmd_again
	btfss ser_con, SER_READY
	goto $-1

	call cmd_dispatch
	goto get_cmd_again


	
;#endif





;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Beacon_start
; Start programming the receiver;
;

beacon_start 

#define MAIN 		0
#define  FREQ_2A 	1
#define  FREQ_1A	2
#define  FREQ_0A	3
#define  FREQ_2B	4
#define  FREQ_1B	5
#define  FREQ_0B	6
#define  FSEP1		7
#define  FSEP0		8
#define	 CURRENT	9
#define	XOSC		0x0A
#define PA_POW		0x0B
#define PLL		0x0c
#define LOCK		0x0D
#define CAL		0x0E
#define MODEM0		0x11
#define FSCTRL		0x13
#define PRESCALER	0x1C

#define F_REG   B'01000000'
#define TX_PD   B'00010000'
#define FS_PD   B'00001000'
#define CORE_PD B'00000100'
#define BIAS_PD B'00000010'
#define RESET_N B'00000001'

#define PA_POWER 0xff
 
;;;;;;;;;;;;;;;;;;;
; initialize and reset: set main
;	F_REG=0			bit6
; 	TX_PD=1;		bit4
;	FS_PD=1;		bit3
; 	CORE_PD=0;		bit2
;	BIAS_PD=1;		bit1
;	reset_N=0;		bit0
	

init1
	bsf cc_pale
	cc1050 MAIN, TX_PD | FS_PD | BIAS_PD

;	reset_N=1;		bit0

	cc1050 MAIN, TX_PD | FS_PD | BIAS_PD | RESET_N

	call	w100mS		; 
;
;	now, program all registers but main;

	cc1050r FREQ_2A, f2
	cc1050r FREQ_1A, f1
	cc1050r FREQ_0A, f0

	cc1050r FREQ_2B, f2
	cc1050r FREQ_1B, f1
	cc1050r FREQ_0B, f0

#if SIMULATE
	goto logmsg
#endif

	cc1050 FSEP1,	0x00
	cc1050 FSEP0, 	0x32
	cc1050 CURRENT, 0x81
	cc1050 XOSC, 	0x00
	cc1050 PA_POW,	0x00
	cc1050 PLL, 	0x48
	cc1050 LOCK, 	0x10
;;	cc1050 CAL, 	x026
	cc1050 MODEM0, 	0x6b	; manchester mode 9.6kb; was 6b
	cc1050 FSCTRL, 	0x01	
	cc1050 PRESCALER, 0x40
	cc1050 0x42, 	0x25


;
; callibrate:
; write cal: cal_dual =1:


; first, turn the outpu power off.

#define CAL_DUAL 	B'01000000'
#define CAL_START 	B'10000000'
#define CAL_WAIT   	B'00100000'
#define CAL_ITERATE	B'00000110'

	cc1050 PA_POW, 0
	cc1050 CAL, CAL_DUAL | CAL_WAIT | CAL_ITERATE
	cc1050 MAIN, RESET_N
	cc1050 CAL, CAL_DUAL | CAL_WAIT | CAL_ITERATE | CAL_START

	call w100mS

	cc1050 CAL, CAL_DUAL | CAL_WAIT | CAL_ITERATE
	
;
; now powerdown;

    	cc1050 MAIN, TX_PD | FS_PD | CORE_PD | BIAS_PD | RESET_N

;
; now, powerup; release CORE_PD
;
	cc1050 MAIN,  TX_PD | FS_PD |  BIAS_PD |RESET_N

	call	w100mS		; wait 100mS 

; release BIAS_PD

	cc1050 MAIN,  TX_PD | FS_PD | RESET_N

	call	w100mS		; wait 100mS 

; release FS_PD
	cc1050 MAIN,  TX_PD |  RESET_N

	call	w100mS		; wait 100mS 

;set output power

	cc1050r PA_POW, power_out;

; release TX_PD
	cc1050 MAIN, RESET_N

here:
	cc1050r PA_POW, power_out;
	cc1050 MAIN, RESET_N

	
        
;;;;;; goto long

long	


	cc1050 MAIN, RESET_N
	;; THis is reset
	call reset
	call	w100mS		; wait 100mS
	;;;
	call poweron
	call calibrate		; calibrate returns with power on

	; We need to do at least one coversion in case we skip both below
	
;
; this call to ad_convert is unnecessary
;
	call ad_convert
   
	call	logmsg		; play ID message
;
; This call to ad_convert is being done with the transmitter enabled
;

	btfsc	options,0	; SKIP playing  voltage if set
	goto check_again

	call ad_convert

	movlw mV
	movwf send
	call morout

	movlw m1
	movwf send
	call morout

        call div_and_morse

check_again

	call poweroff

	btfsc	options,1	; SKIP playing voltage if set
	goto check_done

; let the power drain out 

	call w100mS
	call w100mS
	call w100mS
	call w100mS
	call w100mS

; 
; the last call to ad_convert is the one that will be displayed 
; when a read command is issued.  This call is being  done with the transmitter
; turned off.

	call ad_convert
	call poweron

	movlw mV
	movwf send
	call morout

	movlw m0
	movwf send
	call morout

	call div_and_morse
	call poweroff		; end with the power off

check_done:

; main loop, produce a locating tone for 1 second out of every
; three, then at 5 minute intervals play the ID message

main5	
	movfw	beep_num 	; How many beeps between morse transmissions 
	movwf	d5		; = 300/60 = 5 minutes

fivel

; 
; check for a new command at the beginning of every beep, with the power on
;
try_again3

	banksel ser_con
	btfsc ser_con, SER_READY
	goto cmd_dispatch_h2	; handle charactger
	goto continue23		; no char available

cmd_dispatch_h2
	call cmd_dispatch
	goto try_again3		; try again

continue23
	call poweron

; produce a tone for 1 second, 10 loops of 100mS


	movfw   beep_len
	movwf	delay1

tnlop	movf	beep_freq_loop,w		; get 100mS constant
	movwf	d100		; based upon tone frequency
wtone	bsf	cc_di		; high cycle of tone

	movf	beep_freq,w	; get period constant
	movwf	d250

	movf	beep_freq_h,w	; get period constant
        addlw 1          	; becasue decfsz decs first, then checks
	movwf	d250_h

whight	decfsz	d250,f		; delay high period
	goto	whight

	decfsz d250_h
	goto fixd250
	goto continue11

fixd250
	movlw 0xff
	movwf d250
	goto whight

continue11

	bcf	cc_di		; low cycle of tone
	movf	beep_freq,w	; get period constant
	movwf	d250
	movf	beep_freq_h,w	; get period constant
        addlw 1          	; because decfsz decs first, then checks
	movwf	d250_h

wlowt	decfsz	d250,f		; delay low period
	goto	wlowt

	decfsz d250_h
	goto fixd250a
	goto continue1

fixd250a
	movlw 0xff
	movwf d250
	goto wlowt

continue1
	decfsz	d100,f		; done w/100mS?
	goto	wtone		; nope, loop

	decfsz	delay1,f	; done with 1 second?
	goto	tnlop		; nope, loop

	call poweroff
; delay 1.8 seconds (18 x 100mS)

	movfw   beep_space
	movwf	d250
d15	call	w100mS
	decfsz	d250,f		; done ?
	goto	d15		; nope, loop back

	decfsz	d5,f		; done ?
	goto	fivel		; nope, loop back


;;;;;
UPDATE_FLASH_TIME
        movlw eetime0
	banksel EEADR
	movwf EEADR
	bank00
	movfw time0
	call flash_write

	movlw eetime1
	banksel EEADR
	movwf EEADR
	bank00
	movfw time1
	call flash_write

	movlw eetime2
	banksel EEADR
	movwf EEADR
	bank00
	movfw time2
	call flash_write

	movfw time1
;;;;;;
;  
;
; before we start playing the thing again, let's check the volate and see 
; if we need to shutdown
;
; We've just stopped beeping, then disabled the TX
; then wrote the time to flash
; now do the conversion and check.
;
check_shutdown
	call w100mS	; wait some more for the power to settle down.
	call w100mS	; before doing a conversion
	call w100mS
	call w100mS
	call w100mS

	btfsc	options,2	; if bit2 set, don't check this
	goto long

	call ad_convert

	movfw v_cutoffh
	subwf ADRESULTHI, w
	btfsc status, zerof
	goto doit2		; no match, keep playing
	goto long
doit2

	movfw ADRESULTLO
	subwf v_cutoff, w
    	btfsc status, caryf	
	goto goodnight
        goto long		 ; not less than, keep playing

goodnight
;
; play some ZZZZZ's
;
	call poweron
	movlw mZ
	movwf send
	call morout
	movlw mZ
	movwf send
	call morout
	movlw mZ
	movwf send
	call morout
	movlw mZ
	movwf send
	call morout

	call poweroff

	sleep

;
; if we happen to wake up from sleep, we'll start up again here
;

; this routine sends the message stored at msg in morse

logmsg	clrf	msgreg0
        clrf	msgreg1		; clear the message counter

calldone
	clrf msgreg1
playl	
	movlw callsign	
	addwf msgreg1, w	; w has the address of the char to send
	call	msg		; used to point to msg character
	incf	msgreg1,f	; move to next character (pointer)
				; returned is morse equivalent
play	
	movwf	send		; save the morse character
	movlw	mEND		; is it the end flag?
	subwf	send,w
	btfss	status,zerof
	goto	morplay		; no, so play it
	return			; yes, end, leave

morplay	call morout
	goto playl

;----------------------------------------------------
; morout
;	enter with the morse character to be transmitted
;	in the send register, this routine will toggle
;	an output line, simulating a morse code transmission
;
;
; But, before we do anything, check for a character
;
;
; check for a control word delay flag first, then do
; either the delay or start sending the cw
;
morout	
	btfsc ser_con, SER_READY
	goto cmd_dispatch_h3	; char available
	goto continue99		; no char

cmd_dispatch_h3
	call poweroff
	call cmd_dispatch	; returns if character is bad
	call poweron		; turn power back on just in case we return

continue99
	movlw	mSpace		; is it a space (between words)
	subwf	send,w
	btfss	status,zerof
	goto	morlop		; not a space, go send didah
	call	w100mS		; space, wait 700mS
	call	w100mS		; for 12wpm cw
	call	w100mS		; 400mS here and another
	call	w100mS		; 300mS at mordn
	goto	mordn

; send the cw from the information stored in the send register
; Shift the eight bit register to the left, if the carry bit
; is 0 then send a di, if the carry bit is a 1 send a dah.
; When the send register = 1000 0000 then all bits have been
; sent.

morlop	movlw	0x01		; setup for di
	bcf	status,caryf	; make sure carry bit is clear
	rlf	send,f		; shift to get bit to send
	btfsc	status,caryf	; skip next instruction for di's
	iorlw	0x02		; 3x di = dah
	movwf	didah		; save didah time

morwat	
	movfw   morse_freq_loop
	movwf	d100

wmors	bsf	cc_di	; turn on tone bit (high period)
	movfw   morse_freq
	movwf	d250

	movf    morse_freq_h, w
	addlw 1
	movwf  d250_h

whigh	decfsz	d250,f		; high period delay
	goto	whigh

	decfsz  d250_h
	goto fixd2501
	goto continue12

fixd2501
	movlw 0xff
	movwf d250
	goto whigh

continue12
	bcf	cc_di	; low period for tone bit
	movfw   morse_freq
	movwf	d250

	movf morse_freq_h, w
	addlw 1
	movwf d250_h
	
wlow	decfsz	d250,f		; low period delay
	goto	wlow

	decfsz d250_h
	goto fixd250b
	goto continue13

fixd250b
	movlw 0xff
	movlw d250
	goto wlow

continue13

	decfsz	d100,f		; loop for 100mS
	goto	wmors

	decfsz	didah,f		; fall thru for di's
	goto	morwat		; or do again twice for dah's
	call	w100mS		; delay between di-dahs

	movlw	b'10000000'	; check for end of data
	subwf	send,w
	btfss	status,zerof	; not equal, not end
	goto	morlop		; so continue with this character

mordn	call	w100mS		; delay between characters
	call	w100mS		; 300mS for 12wpm
	call	w100mS
;	incf	msgreg1,f	; move to next character (pointer)
;	goto	playl		; loop for rest of message
	return


;-----------------------------------------------------
; 100 mS delay rtn 


w500mS	
#if SIMULATE
	return
#endif

	call w100mS
	call w100mS
	call w100mS
	call w100mS
	return

w100mS  
#if SIMULATE
	return
#endif
	movlw   d'100'          ; 1 cycle 
        movwf   delay2          ; 1 cycle 

                                ; gets kinda funky here 
                                ; as there is a 1mS rtn 
                                ; built into the 100mS 
                                ; rtn (saving a sub call 
                                ; and stack space!) 

d100ms  movlw   d'197'          ; 1 cycle           ---+ 
        movwf   delay1          ; 1 cycle              | 
d1ms2   goto    $+1             ; 2 cycles (x 197)     | 
        decfsz  delay1,f        ; 1 cycle  (x 197)     | 
        goto    d1ms2           ; 2 cycles (x 197)     | 
                                ; 3 + 197 x 5 = 988    | 
        goto    $+1             ; = 990                |  
        goto    $+1             ; = 992             ---+ 

        decfsz  delay2,f        ; 1 cycle 
        goto    d100ms          ; 2 cycles, = 995 

; setup cycles 2 + 1 for fall thru = 3 
; interior cycles = 100 x 995 = 99500 
; = 99503 uS (99.503 mS) 
; the calling rtn eats up 2 more and the return eats 
; two cycles, = 99507uS, need to eat up 493 uS to 
; give a precise 100mS delay 
; also gives us an approximate 500uS delay

w500uS  movlw   d'122'          ; 1 cycle 
        movwf   delay1          ; 1 cycle 
d101ms  nop                     ; 1 cycle x 122 
        decfsz  delay1,f        ; 1 cycle x 122 
        goto    d101ms          ; 2 cycles x 122 

; 2 + 1 + 4 x 122 = 491 uS 

        goto    $+1             ; waste 2 more cycles 
                                ; for 493 uS, add in 
                                ; the call (2uS) and 
                                ; return (2uS) for 
        return                  ; a grand total of 100mS 

w250uS  movlw   d'65'          ; 1 cycle 
        movwf   delay1          ; 1 cycle 
d101ams nop                     ; 1 cycle x 122 
        decfsz  delay1,f        ; 1 cycle x 122 
        goto    d101ams          ; 2 cycles x 122 



        goto    $+1             ; waste 2 more cycles 
                                ; for 493 uS, add in 
                                ; the call (2uS) and 
                                ; return (2uS) for 
        return                  ; a grand total of 100mS 

w125uS  movlw   d'32'          ; 1 cycle 
        movwf   delay1          ; 1 cycle 
d101bms nop                     ; 1 cycle x 122 
        decfsz  delay1,f        ; 1 cycle x 122 
        goto    d101bms          ; 2 cycles x 122 

        goto    $+1             ; waste 2 more cycles 
                                ; for 493 uS, add in 
                                ; the call (2uS) and 
                                ; return (2uS) for 
        return                  ; a grand total of 100mS 



; equates for morse characters 
; specify the coded characters by using 'm' followed 
; by the letter you want to send the coded form of 
; morse is based in... 0 = dit, 1 = dash, roll out 
; the bits, when 0x80 remains the code is done 
; this Morse Code storage rtn was published in _BYTE_ 
; October 1976, pg. 36 in an article by L. Krakauer 

mA      equ     0x60 
mB      equ     0x88 
mC      equ     0xa8 
mD      equ     0x90 
mE      equ     0x40 
mF      equ     0x28 
mG      equ     0xd0 
mH      equ     0x08 
mI      equ     0x20 
mJ      equ     0x78 
mK      equ     0xb0 
mL      equ     0x48 
mM      equ     0xe0 
mN      equ     0xa0 
mO      equ     0xf0 
mP      equ     0x68 
mQ      equ     0xd8 
mR      equ     0x50 
mS      equ     0x10 
mT      equ     0xc0 
mU      equ     0x30 
mV      equ     0x18 
mW      equ     0x70 
mX      equ     0x98 
mY      equ     0xb8 
mZ      equ     0xc8 

m0      equ     0xfc 
m1      equ     0x7c 
m2      equ     0x3c 
m3      equ     0x1c 
m4      equ     0x0c 
m5      equ     0x04 
m6      equ     0x84 
m7      equ     0xc4 
m8      equ     0xe4 
m9      equ     0xf4 

mPeriod equ     0x56 
mComma  equ     0xce 
mQuest  equ     0x32 
mEqual  equ     0x8c 
mColon  equ     0xe2 
mSemi   equ     0xaa 
mSlash  equ     0x94 
mDash   equ     0x86 

mSpace  equ     0x01    ; space between words 
mEND    equ     0x00    ; end of the info 


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; write_addr_data:
;
; Write the address and data to the cc1050
; 
; there are only 6 address bits
; MK

write_addr_data:
	bsf cc_pclk
	bcf cc_pdata

	bcf cc_pale

	btfsc	outaddr, 6		; test bit 6, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outaddr, 5		; test bit 5, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outaddr, 4		; test bit 4, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outaddr, 3		; test bit 3, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outaddr, 2		; test bit 2, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outaddr, 1		; test bit 1, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outaddr, 0		; test bit 0, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

; this is a write, so set  to 1, and toggle clk.
	bsf cc_pdata
	bcf cc_pclk
	bsf cc_pclk

; and raise ale

	bsf cc_pale	

	bcf	cc_pdata		;clear the data line

	btfsc	outdata, 7		; test bit 7, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outdata, 6		; test bit 6, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outdata, 5		; test bit 5, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outdata, 4		; test bit 4, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outdata, 3		; test bit 3, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outdata, 2		; test bit 2, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outdata, 1		; test bit 1, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	btfsc	outdata, 0		; test bit 0, skip of 0
	bsf 	cc_pdata		; set the data line
	bcf	cc_pclk			; set the clock line
	bsf 	cc_pclk			; clear the clock line
	bcf	cc_pdata		;clear the data line

	retlw 	0

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; poweron

poweron:
	cc1050 MAIN,  TX_PD | FS_PD |  BIAS_PD |RESET_N
	call	w100mS		; wait 100mS 

	cc1050 MAIN,  TX_PD | FS_PD | RESET_N
	call	w100mS		; wait 100mS 

	cc1050 MAIN,  TX_PD |  RESET_N
	call	w100mS		; wait 100mS 

; release TX_PD
	cc1050 MAIN, RESET_N
	cc1050r PA_POW, power_out;
	retlw 0

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; poweroff

poweroff:
	cc1050 MAIN, TX_PD | FS_PD | BIAS_PD |CORE_PD  | RESET_N
	cc1050 PA_POW, 0x00
	retlw 0

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; reset

reset:	 
        cc1050 MAIN, TX_PD | FS_PD | CORE_PD | BIAS_PD 
	call	w100mS		; wait 100mS 
        cc1050 MAIN, TX_PD | FS_PD | CORE_PD | BIAS_PD | RESET_N;
	cc1050r FREQ_2A, f2
	cc1050r FREQ_1A, f1
	cc1050r FREQ_0A, f0


	cc1050r FREQ_2B, f2
	cc1050r FREQ_1B, f1
	cc1050r FREQ_0B, f0


	cc1050 FSEP1,	0x00
	cc1050 FSEP0, 	0x32
	cc1050 CURRENT, 0x81
	cc1050 XOSC, 	0x00
	cc1050 PA_POW,	0x00
	cc1050 PLL, 	0x48
	cc1050 LOCK, 	0x10
;;	cc1050 CAL, 	x026
	cc1050 MODEM0, 	0x6b	; manchester mode 9.6kb; was 6b
	cc1050 FSCTRL, 	0x01	
	cc1050 PRESCALER, 0x40
	cc1050 0x42, 	0x25
	retlw 0

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; calibrate

calibrate:
	cc1050 PA_POW, 0
	cc1050 CAL, CAL_DUAL | CAL_WAIT | CAL_ITERATE
	cc1050 MAIN, RESET_N
	cc1050 CAL, CAL_DUAL | CAL_WAIT | CAL_ITERATE | CAL_START

	call w100mS

	cc1050 CAL, CAL_DUAL | CAL_WAIT | CAL_ITERATE
	cc1050r PA_POW, power_out;
	retlw 0

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; calibrate

output_crlf
	call serout_crlf
	bank00
	return

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; output_model

output_model
	clrf i
	loop
 	  movfw i
	  call get_model
	  andlw 0xff
	  btfsc STATUS, Z
	  break 
	  call ser_out
	  bank00
	  incf i, f
	endloop
	return

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; output_model

output_version
	clrf i
	loop
 	  movfw i
	  call get_version
	  andlw 0xff
	  btfsc STATUS, Z
	  break 
	  call ser_out
	  bank00
	  incf i, f
	endloop

	return

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; read_program

read_program
	bank0	;;;;; was bank1
	movlw callsign
	movwf FSR

	for k, 0, program_length-1
	  movfw k
	  banksel EEADR
	  movwf	EEADR
	  bank00
	  call flash_read
	  movwf INDF
	  incf FSR, f
	next
	return

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; write_serial

write_serial
	movlw callsign
	movwf FSR

	for k, serialnum, serialnum+9
	  movfw k
	  banksel	EEADR
	  movwf	EEADR
	  bank00
	  movfw INDF
	  call flash_write
	  incf FSR, f
	next
	return	

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; write_program
;

write_program
	movlw callsign
	movwf FSR

	for k, 0, program_pgm_length - 1
	  movfw k
	  banksel	EEADR
	  movwf	EEADR
	  bank00
	  movfw INDF
	  call flash_write
	  incf FSR, f
	next
	return

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; ad_convert
	  
ad_convert
	BANKSEL ANSEL
	MOVLW 0x10
	MOVWF ANSEL			; AN4 (C0) is an analog input

	BANKSEL ADCON1
	MOVLW B'00010000'		
	MOVWF ADCON1

	BANKSEL ADCON0
	MOVLW B'10010001'	; RIght, VDD Vref, AN4, Start
	MOVWF ADCON0
	CALL pause_1ms		; really only needs to be 11.5 uSecs
	BSF ADCON0, GO		; Start the conversion
	BTFSC ADCON0, GO
	GOTO $-1

	BANKSEL ADRESH
	MOVF ADRESH, W
;	CLRF ADRESH

	BANKSEL ADRESULTHI
	MOVWF ADRESULTHI

	BANKSEL ADRESL
	MOVF ADRESL, W
;	CLRF ADRESL

	BANKSEL ADRESULTLO
	MOVWF ADRESULTLO
;
; now turn it off
;
	BANKSEL ADCON0
	MOVLW B'10010000'	; RIght, VDD Vref, AN4, Start
	MOVWF ADCON0

	BANKSEL ANSEL
	CLRF ANSEL

	BANKSEL ADCON0
	CLRF ADCON0

	BANKSEL ADCON1
	CLRF ADCON1

	BANKSEL  ADRESULTLO

	return

#define bf_carry 3, 0
#define bf_zero 3, 2

#define same 1
#define wreg 0

#define stc bsf bf_carry
#define clc bcf bf_carry

;-[ Div ]--------------------------------------------------------------
; Call w/: Number in f_divhi:f_divlo, divisor in W.
; Returns: Quotient in f_divlo, remainder in f_divhi. W preserved.
;          Carry set if error. Z if divide by zero, NZ if divide overflow.
; Notes:   Works by left shifted subtraction.
;          Size = 29, Speed(w/ call&ret) = 7 cycles if div by zero
;          Speed = 94 minimum, 129 maximum cycles

	
Div:
    addlw 0          ; w+=0 (to test for div by zero)
    stc              ; set carry in case of error
    btfsc bf_zero    ; if zero
    return          ;   return (error C,Z)

    call DivSkipHiShift
iDivRepeat = 8
    while iDivRepeat

    call DivCode

iDivRepeat--
    endw

    rlf f_divlo, same ; C << lo << C

    ; If the first subtract didn't underflow, and the carry was shifted
    ; into the quotient, then it will be shifted back off the end by this
    ; last RLF. This will automatically raise carry to indicate an error.
    ; The divide will be accurate to quotients of 9-bits, but past that
    ; the quotient and remainder will be bogus and carry will be set.

    bcf bf_zero  ; NZ (in case of overflow error)
    return       ; we are done!

DivCode
    rlf f_divlo, same    ; C << lo << C
    rlf f_divhi, same    ; C << hi << C
    btfss bf_carry       ; if Carry
    goto DivSkipHiShift ;
    subwf f_divhi, same  ;   hi-=w
    stc                  ;   ignore carry
    return               ;   done
                         ; endif
DivSkipHiShift
    subwf f_divhi, same  ; hi-=w
    btfsc bf_carry       ; if carry set
    return              ;   done
    addwf f_divhi, same  ; hi+=w
    clc                  ; clear carry
    return               ; done


MPY8X8:

	CLRF f_divhi
	CLRF i
	BSF i,3
	RRF f_divlo

LOOPmult:

	SKPNC
	ADDWF f_divhi
	
	RRF f_divhi
	RRF f_divlo

	DECFSZ i
	GOTO LOOPmult
	return

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; div_and_morse
;
; change the AD value into decimal by dividing my some magic numbers,
; and the transmit these values;
;
; more magic numbers
;

div_and_morse:


	movlw mSpace
	movwf send
	call morout

	movfw ADRESULTHI
	movwf f_divhi

	movfw ADRESULTLO
	movwf f_divlo

	movlw D'171'
	call Div		; now quotent is in low, remainder in hi

	movfw f_divlo
	call msgnum
	movwf	send		; save the morse character
	call morout

	movlw mPeriod
	movwf send
	call morout

	movfw f_divhi
	movwf f_divlo
	clrf f_divhi
	movlw D'10'
	call MPY8X8
	movlw D'171
	call Div

	movfw f_divlo
	call msgnum
	movwf	send		; save the morse character
	call morout

	movfw f_divhi
	movwf f_divlo
	clrf f_divhi
	movlw D'10'
	call MPY8X8
	movlw D'171'
	call Div

	movfw f_divlo
	call msgnum
	movwf	send		; save the morse character
	call morout

	return

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; TImer is ticking once every xxx secsonds timer counts up;
; timer interrupts when it wraps from FFFF-> 0.
;
; clock speed / 2 = tick rate
; pre-scaler is 8, so 4Mhz/2/8 = 125,000 Hz.
; 
; that means  one cycle from 0-> FFFF (65536)
; and anther cycle from 17b8-> FFFF   (another 59464) 
; for a total of 125000 ticks of the clock.
;
; I think it should really be 17B8, not 28B8
; 
; When timer expires,  inc timer, which  really  consist of 3 
; seperate 8 bit counters .
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

clkint				
	
	ifbc clk_state, 0
	  bsf clk_state, 0
	  movlw 0xb8
	  movwf TMR1L
	  movlw 0x17
	  movwf TMR1H
	  return
	endif_

	ifbs clk_state, 0
	  bcf clk_state, 0
	  incf time0
	  ifbs status, zerof
	    incf time1
	    ifbs status, zerof
	      incf time2
	      return
	    endif_
	    return
	  endif_
	  return
	endif_

	;; should never get here.....
	sleep


	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
pause_1ms
#ifdef debug
	return
#endif
	from pause_i, .199
	goto $+1
	endfrom

	return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

radix dec

#define SERIN_DEC
#define SEROUT_DEC
#define BAUD_RATE 2400
#include "serutil.asm"
#include "serial.asm" 
#include "flash.asm"


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        ORG 0x2100
eecall
	de  mC, mQ, mSpace, mC, mQ, mSpace
	de mEND

        ORG 0x2120
eef2	de 0x42		; these #'s correspond to 433.92
eef1	de 0x14
eef0	de 0x25
eepower_out		de 0xff
eebeep_tone		de 0		; freq of beep
eebeep_len 		de 3 	        ; number of beeps between call signs
eebeep_num		de 50		; length of beep in 100's of ms
eebeep_space		de 45		; time between beeps (in 100ms increments)
eebeep_freq_h		de 0x00		; Beep Freqency
eebeep_freq		de 0xA6		; Beep Freqency
eebeep_freq_loop_h 	de 0x00
eebeep_freq_loop 	de 0x64
eemorse_freq_h 		de 0
eemorse_freq 		de 0xa6
eemorse_freq_loop 	de 0x64
eebeep_space_lv		de 0		; time between beeps when voltage is low
eeoptions		de 0xf		; both TX on and TXOFF voltages ID'd
eev_cutoffh		de 2		; trigger for  low V operation
eev_cutoff		de 0x44		; trigger for low V operation	244 is 1.7volts (3.4 VOls)
eetime2			de 0x00
eetime1			de 0x00
eetime0			de 0x00

junk			de 0xfe
junk2			de 0xfe
junk3			de 0x5a, 0x5a, 0x5a, 0x5a

;
; Private data follows
;
serialnum		de   0x00, 0x00, 0x00, 0x00
scall 	     		de  '?','?','?','?','?','?'

	end
